# bu-lib.pm  -  Library functions for bu
# Copyright (C) 2002  Vincent Stemen <bu@hightek.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: bu_lib.pm,v 1.1 2002/05/08 03:59:24 vince Exp vince $


sub errmsg
{
  my $string = shift;

  (! $string) && return;
  print STDERR "$string";
  print ERRLOG "$string";
}


# fgrep()
# Usage: fgrep($expression, $file)
# Returns a list of lines in file that match $expression

sub fgrep($$)
{
  my ($expr, $file) = @_;
  my @lines = ();

  open(FILE, "$file") || die "Cannot open $file\n$!";
  while (<FILE>) {
    /$expr/ && push(@lines, $_);
    }
  close(FILE);
  return(@lines);
}


sub eject_cd()
{
  if ($SYS_TYPE =~ /BSD/) {
    system("cdcontrol -f $DEVICE eject");
    }
}


# close_cd()
# Usage: close_cd([delay])
# Delay is optional.  It defaults to 10 seconds delay before returning to
# allow time for the drive to become ready after closing.

sub close_cd(;$)
{
  my $sec = shift;
  my $seconds = defined($sec) ? $sec : 10;

  if ($SYS_TYPE =~ /BSD/) {
    system("cdcontrol -f $DEVICE close");
    }

  sleep($seconds);  # Allow time for the drive to become ready
}

# resetcd()
# Usage: resetcd([delay])
# Opens and closes the CD tray.
# Delay is optional.  It defaults to 10 seconds delay before returning to
# allow time for the drive to become ready after closing.

sub reset_cd(;$)
{
  my $seconds = shift;
  print "Reseting CD writer\n";
  eject_cd();
  close_cd($seconds);
}


sub blank_cd()
{
  my $testopt = "";
  my @blankcd = (`whoami` =~ /^root$/) ? qw(nice --5) : "";

  if ($SYS_TYPE =~ /BSD/)
    {
    $Test && ($testopt = "-t");
    push(@blankcd, "burncd", $testopt, "-s", $SPEED, "-f", $DEVICE, "blank");
    if (system("@blankcd"))
      {
      sleep(1);
      system("cdcontrol -f $DEVICE reset");
      return(0);
      }
    }

  return(1);
}


sub mount_cd()
{
  my $fstype;

  if ($SYS_TYPE =~ /BSD/) {
    $fstype = "cd9660";
    }
  elsif ($SYS_TYPE eq "Linux") {
    $fstype = "iso9660";
    }

  if (system("mount -t $fstype $DEVICE $CD_MNT"))
    {
    warn "Cannot mount $DEVICE to $CD_MNT";
    return(0);
    }
  else {
    return(1);
    }
}


sub record_time()
{
  open(T, ">$LOCAL_TIME"); print T time(); close(T);
}


sub wait_for_cd()
{
  $wait_time = 11;   # Seconds to wait for the drive to become ready after
                     # the door closes.

  if (-f $LOCAL_TIME)
    {
    open(T, "$LOCAL_TIME"); $pretime = <T>; close(T);
    $curtime = time();
    $elapsed = $curtime - $pretime;
    ($elapsed < $wait_time) && sleep($wait_time - $elapsed);
    }

  unlink($LOCAL_TIME);
}


sub mkfifo($)
{
  my $fifo = shift;

  if (! -p $fifo && system("mkfifo --mode 600 $fifo")) {
    die "Cannot create fifo $fifo\n$!";
    }
}


# run()
# Run the specified system command and exit with the return code if
# anything other than zero is returned from the command.

sub run(@)
{
  my $rc = system("@_") >> 8;
  if ($rc) {
    exit($rc);
    }
  else {
    return(0);
    }
}


# read_file()
# Usage: @lines = read_file($filename)
# Returns the list of lines in the file.
# If the file does not exist, it waits several fractions of a second for
# the file to appear before giving up and returning 0.

sub read_file($)
{
  my $file = shift;
  my @data;

  for (1,2,3,4)
    {
    if (-f $file)
      {
      open(FILE, "$file") || next;
      @data = <FILE>;
      close(FILE);
      return(@data);
      }
    else {
      system("usleep 75000");
      }
    }

  return(0);
}


sub signal
{
  my $signal = shift;

  # Disable further signals to make sure I can properly clean up
  $SIG{INT}  = "";
  $SIG{TERM} = "";

  if ($signal eq "INT") { bye($RC_SIGINT) }
  elsif ($signal eq "TERM") { bye($RC_SIGTERM) }
}


sub write_file($@)
{
  my ($file, @data) = @_;
  open(FILE, ">>$file") || die "Cannot write to $file\n$!";
  print FILE @data;
  close(FILE);
}


$ERRLOG = "$LOG/errors";
open(ERRLOG, ">$ERRLOG") || die "Cannot write to $LOG/errors\n$!\n";
$SIG{INT}  = \&signal;
$SIG{TERM} = \&signal;

1;
